# This file was autogenerated! Do not modify!
# set interfacing Set.h

from .defs cimport *
from .pm_object cimport PmObject

cdef class SetInt(PmObject):
    cdef pm_Set[int] pm_obj

cdef class SetMatrixRational(PmObject):
    cdef pm_Set[pm_Matrix[pm_Rational]] pm_obj

cdef class SetArrayInt(PmObject):
    cdef pm_Set[pm_Array[int]] pm_obj

cdef class SetSetInt(PmObject):
    cdef pm_Set[pm_Set[int]] pm_obj
