# This file was autogenerated! Do not modify!
# quadratic_extension interfacing QuadraticExtension.h

from .defs cimport *
from .pm_object cimport PmObject

cdef class QuadraticExtension(PmObject):
    cdef pm_QuadraticExtension pm_obj
