# This file was autogenerated! Do not modify!
# power_set interfacing PowerSet.h

from .defs cimport *
from .pm_object cimport PmObject

cdef class PowerSetInt(PmObject):
    cdef pm_PowerSet[int] pm_obj
