# This file was autogenerated! Do not modify!
# sparse_matrix interfacing SparseMatrix.h

from .defs cimport *
from .pm_object cimport PmObject

cdef class SparseMatrixInt(PmObject):
    cdef pm_SparseMatrix[int] pm_obj

cdef class SparseMatrixRational(PmObject):
    cdef pm_SparseMatrix[pm_Rational] pm_obj
