# This file was autogenerated! Do not modify!
# array interfacing Array.h

from .defs cimport *
from .pm_object cimport PmObject

cdef class ArrayPairStringString(PmObject):
    cdef pm_Array[pair[string, string]] pm_obj

cdef class ArraySetArrayInt(PmObject):
    cdef pm_Array[pm_Set[pm_Array[int]]] pm_obj

cdef class ArrayArrayString(PmObject):
    cdef pm_Array[pm_Array[string]] pm_obj

cdef class ArrayPairStringArrayString(PmObject):
    cdef pm_Array[pair[string, pm_Array[string]]] pm_obj

cdef class ArrayString(PmObject):
    cdef pm_Array[string] pm_obj

cdef class ArrayRational(PmObject):
    cdef pm_Array[pm_Rational] pm_obj

cdef class ArrayPowerSetInt(PmObject):
    cdef pm_Array[pm_PowerSet[int]] pm_obj

cdef class ArraySetInt(PmObject):
    cdef pm_Array[pm_Set[int]] pm_obj

cdef class ArrayInt(PmObject):
    cdef pm_Array[int] pm_obj

cdef class ArrayBool(PmObject):
    cdef pm_Array[bool] pm_obj

cdef class ArrayMatrixInteger(PmObject):
    cdef pm_Array[pm_Matrix[pm_Integer]] pm_obj

cdef class ArrayArrayInt(PmObject):
    cdef pm_Array[pm_Array[int]] pm_obj

cdef class ArrayArrayPairStringString(PmObject):
    cdef pm_Array[pm_Array[pair[string, string]]] pm_obj
