# This file was autogenerated! Do not modify!
# rational_function interfacing RationalFunction.h

from .defs cimport *
from .pm_object cimport PmObject

cdef class RationalFunctionRationalInt(PmObject):
    cdef pm_RationalFunctionRationalInt pm_obj
