# This file was autogenerated! Do not modify!
# incidence_matrix interfacing IncidenceMatrix.h

from .defs cimport *
from .pm_object cimport PmObject

cdef class IncidenceMatrixNonSymmetric(PmObject):
    cdef pm_IncidenceMatrixNonSymmetric pm_obj
