# This file was autogenerated! Do not modify!
# polynomial interfacing Polynomial.h

from .defs cimport *
from .pm_object cimport PmObject

cdef class UniPolynomialRationalInt(PmObject):
    cdef pm_UniPolynomialRationalInt pm_obj
