# This file was autogenerated! Do not modify!
# map interfacing Map.h

from .defs cimport *
from .pm_object cimport PmObject

cdef class MapRationalRational(PmObject):
    cdef pm_Map[pm_Rational,pm_Rational] pm_obj

cdef class MapStringString(PmObject):
    cdef pm_Map[string,string] pm_obj

cdef class MapIntInt(PmObject):
    cdef pm_Map[int,int] pm_obj

cdef class MapIntegerInt(PmObject):
    cdef pm_Map[pm_Integer,int] pm_obj
