# This file was autogenerated! Do not modify!
# vector interfacing Vector.h

from .defs cimport *
from .pm_object cimport PmObject

cdef class VectorFloat(PmObject):
    cdef pm_Vector[float] pm_obj

cdef class VectorInteger(PmObject):
    cdef pm_Vector[pm_Integer] pm_obj

cdef class VectorQuadraticExtension(PmObject):
    cdef pm_Vector[pm_QuadraticExtension] pm_obj

cdef class VectorRational(PmObject):
    cdef pm_Vector[pm_Rational] pm_obj

cdef class VectorInt(PmObject):
    cdef pm_Vector[int] pm_obj
